from __future__ import print_function

import base64
import mutagen
import mutagen.mp3
import shutil
import os
import subprocess

from django.core.management.base import BaseCommand, CommandError

from ...models import SoundFile


class Command(BaseCommand):

    def add_arguments(self, parser):
        parser.add_argument(
            '--force',
            action='store_true',
            dest='force',
            default=False,
            help='Create files even if they exist')
        parser.add_argument('--reset-metadata',
            action='store_true',
            dest='reset_metadata',
            default=False,
            help='Reset metadata on all files')
        parser.add_argument('--emission',
            dest='emission',
            metavar='EMISSION',
            default=None,
            help='Process files belonging to emission only')
        parser.add_argument('--episode',
            dest='episode',
            metavar='EPISODE',
            default=None,
            help='Process files belonging to episode only')
        parser.add_argument('--copy',
            action='store_true',
            dest='copy',
            default=False,
            help='Copy initial file (ogg only)')

    def handle(self, force, reset_metadata, copy, emission, episode, verbosity, **kwargs):
        self.verbose = (verbosity > 1)
        self.copy = copy

        for soundfile in SoundFile.objects.select_related().exclude(podcastable=False):
            if emission and soundfile.episode.emission.slug != emission:
                continue
            if episode and soundfile.episode.slug != episode:
                continue
            try:
                if soundfile.file is None or not os.path.exists(soundfile.file.path):
                    continue
            except ValueError:  # no file associated with it
                continue
            for format in ('ogg', 'mp3'):
                file_path = soundfile.get_format_path(format)
                created = False
                if not os.path.exists(file_path) or force:
                    created = self.create(soundfile, format)
                if created or reset_metadata:
                    self.set_metadata(soundfile, format)
            if (force or not soundfile.duration) and os.path.exists(soundfile.get_format_path('ogg')):
                cmd = ['soxi', '-D', soundfile.get_format_path('ogg')]
                soundfile.duration = int(float(subprocess.check_output(cmd)))
                soundfile.save()


    def create(self, soundfile, format):
        file_path = soundfile.get_format_path(format)
        if not os.path.exists(os.path.dirname(file_path)):
            os.mkdir(os.path.dirname(file_path))

        cmd = ['ffmpeg', '-y', '-i', soundfile.file.path]

        if format == 'ogg':
            if self.copy:
                shutil.copy(soundfile.file.path, file_path)
                return
            cmd.extend(['-q:a', '5'])
        elif format == 'mp3':
            cmd.extend(['-q:a', '4'])

        cmd.append(file_path)

        if self.verbose:
            print('creating', file_path)
            print('  ', ' '.join(cmd))
        else:
            cmd[1:1] = ['-loglevel', 'quiet']

        subprocess.call(cmd)
        return os.path.exists(file_path)

    def set_metadata(self, soundfile, format):
        file_path = soundfile.get_format_path(format)

        audio = mutagen.File(file_path, easy=True)

        if 'comment' in audio:
            del audio['comment']
        if soundfile.fragment is True and soundfile.title:
            audio['title'] = '%s - %s' % (
                    soundfile.episode.title,
                    soundfile.title)
        else:
            audio['title'] = soundfile.episode.title
        audio['album'] = soundfile.episode.emission.title
        audio['artist'] = 'Radio Panik'

        if soundfile.episode.image or soundfile.episode.emission.image:
            image = (soundfile.episode.image or soundfile.episode.emission.image)
            image.file.open()
            if os.path.splitext(image.path)[1].lower() in ('.jpeg', '.jpg'):
                mimetype = 'image/jpeg'
            elif os.path.splitext(image.path)[1].lower() == '.png':
                mimetype = 'image/png'
            else:
                mimetype = None
            if mimetype:
                if format == 'ogg':
                    audio['coverartmime'] = mimetype
                    audio['coverartdescription'] = 'image'
                    audio['coverart'] = base64.encodebytes(image.read()).replace(b'\n', b'').decode('ascii')
                elif format == 'mp3':
                    audio.save()
                    audio = mutagen.mp3.MP3(file_path)
                    audio.tags.add(mutagen.id3.APIC(
                            encoding=3, description='image',
                            type=3, mime=mimetype, data=image.read()))
            image.close()

        audio.save()
